#!/usr/bin/perl
###########################################################################
#    WWWThreads  - Interactive Discussion Forum
#    Wired Community Software
#    Rick Baker rbaker@wcsoft.net 
#    (c) Copyright 1999-2000 
###########################################################################
  use w3ttheme qw(%theme);
  use w3tvars qw(%config);
  use w3t qw($dbh %lang);
  use strict;

# --------------------
# Assign the variables
  my %FORM   = w3t::get_input;
  my $Cat    = $FORM{'Cat'};
  my $Board  = $FORM{'Board'};
  my $page   = $FORM{'page'};
  my $view   = $FORM{'view'};
  my $sb     = $FORM{'sb'};
  my $sort   = $FORM{'sort'};

# -----------------------------------------------------------------
# Grab the cookie so we can pass the username and password if found
  my %cookie   = w3t::get_cookie();
  my $Username = $cookie{'Username'};
  my $Password = $cookie{'Password'};

# -----------------------
# Connect to the database
  w3t::db_connect();

# -----------------
# Get the user info 
  my @user = w3t::authenticate($Username,$Password,"U_Username, U_Password, U_Groups");
  my %user;
  $user{'Username'} = $user[0];
  $user{'Password'} = $user[1];
  $user{'Groups'}   = $user[2]; 
  $user{'FontSize'} = $user[3];
  $user{'FontFace'} = $user[4];
  $user{'Status'}   = $user[5];

# ---------------
# send the header
  w3t::send_header("$config{'title'}",$Cat,0,$Username,$Password,0,%user);

# ------------------------------------------------------------------------
# If they are logged in then we check their groups, otherwise they
# get set to the guest group 
  my $Groups = $user{'Groups'};
  $Groups  ||= "-4-";
   
# -----------------------------------------------------------------
# If we have a Cat variable then we need to set it up for the query
  my $catonly;
  if ($Cat) {
    $catonly = $Cat;
    $catonly =~s/,/ OR Cat_Number = /igs;
    $catonly = "WHERE Cat_Number = ".$catonly;
  }

# ----------------------------------------------------------------------
# Lets see how many total registered users there are but only if we want
# to display this information
  my $showreg;
  if ($config{'showreg'}) {
    my $query = qq!
      SELECT COUNT(*)
      FROM   w3t_Users
    !;
    my $stb = $dbh->prepare ($query) or w3t::not_right("Query syntax error: $DBI::errstr. Query: $query",$Cat,1);
    $stb -> execute()                or w3t::not_right("Can't execute query: $query. Reason: $DBI::errstr",$Cat,1);
    my ($registered) = $stb -> fetchrow_array;
    $stb -> finish;
    $showreg = "$registered $lang{'REGED_USERS'}";
  }

# -----------------------------------------------------------------------
# Print the html for the number of registered users but only display if we
# want to 
  print qq~
    <table border="0" width="$theme{'tablewidth'}" align="$theme{'tablealign'}" cellpadding="0" cellspacing="0">
    <tr><td align="left">
    $showreg
    </td><td align="right">
  ~;

# ------------------------------------------------
# Lets see how many new private messages they have
  if ($Username) {
    my $Username_q = $dbh -> quote($Username);
    my $query = qq!
      SELECT COUNT(*) 
      FROM   w3t_Messages
      WHERE  M_Status   = 'N'
      AND    M_Username = $Username_q
    !;
    my $stm = $dbh -> prepare($query) or w3t::not_right("Can't prepare $query.  Reason: $DBI::errstr.",$Cat,1);
    $stm -> execute()                 or w3t::not_right("Can't execute $query.  Reason: $DBI::errstr.",$Cat,1);
    my $messages = $stm -> fetchrow_array;
    $stm -> finish;
    if ( ($messages < 1) || ($messages > 1) ) {
      print "$lang{'INTRO_SUB'}, $Username.  $lang{'WEL_PRIV2'}$messages $lang{'WEL_PRIV3'}";
    }
    elsif ( ($messages > 0) && ($messages < 2) ) {
      print "$lang{'INTRO_SUB'}, $Username.  $lang{'WEL_PRIV2'}1 $lang{'WEL_PRIV3'}";
    }
  } 
  else {
    print "$lang{'WEL_PRIV_NO'}";
  }
  print "</td></tr></table>";

# --------------------------
# Get the list of categories
  my $query = qq!
    SELECT DISTINCT Cat_Title,Cat_Number
    FROM w3t_Category
    $catonly
    ORDER BY Cat_Number
  !;
  my $sta = $dbh->prepare ($query) or w3t::not_right("Query syntax error: $DBI::errstr. Query: $query",$Cat,1);
  $sta -> execute() or w3t::not_right("Can't execute query: $query. Reason: $DBI::errstr",$Cat,1);
  my $j = 0;

# ----------------------------
# Cycle through the categories
  while (my ($CatTitle,$CatNumber) = $sta -> fetchrow_array) {

  # ---------------------------------------
  # Get the list of boards in this category
    my $CatTitle_q = $dbh -> quote($CatTitle);
    my $NULL_q     = $dbh -> quote("NULL");
    my $query = qq!
      SELECT   Cat_Keyword,
               Cat_Sorter,
               COUNT(*) 
      FROM     w3t_Category
      WHERE    Cat_Title   = $CatTitle_q
      AND      Cat_Keyword <> $NULL_q
      GROUP BY Cat_Keyword,Cat_Sorter
      ORDER BY Cat_Sorter 
    !;
    my $stb = $dbh->prepare ($query) or w3t::not_right("Query syntax error: $DBI::errstr. Query: $query",$Cat,1);
    $stb -> execute()                or w3t::not_right("Can't execute query: $query. Reason: $DBI::errstr",$Cat,1);

  # ------------------------------------------------------------
  # This variable let's us know if there were any boards in this
  # category that were visible to the user.  As soon as we come
  # across a board that they can see, it gets set to "yes"
    my $table_done = "no";
    my $k=0;

  # --------------------------------------------------------------
  # We need to format a SQL query to see what boards this user can
  # view
    my @Groups = split(/-/,$Groups);
    my $groupquery = "AND (";
    for my $i(0 .. $#Groups) {
      $Groups[$i] !~/[0123456789]/ and next;
      $i > 1 and $groupquery .=" OR ";
      $groupquery .= "Bo_Read_Perm LIKE '%-$Groups[$i]-%'";
    }
    $groupquery .= ")";

  # ---------------------------------------------
  # Now cycle through the boards in this category
    while (my ($CatKeyword,$Sorter,$BoardCount) = $stb -> fetchrow_array) {
      $k++;
      if(!$CatKeyword) { next; } 

      my $Keyword_q = $dbh -> quote($CatKeyword);
      my $query = qq!
        SELECT Bo_Title,Bo_Description,Bo_Keyword,Bo_Total,Bo_Last,Bo_Number,Bo_Moderated,Bo_Read_Perm,Bo_Write_Perm
        FROM   w3t_Boards
        WHERE  Bo_Keyword = $Keyword_q
        $groupquery
      !;
      my $sth = $dbh->prepare ($query) or w3t::not_right("Query syntax error: $DBI::errstr. Query: $query",$Cat,1);
      $sth -> execute()                or w3t::not_right("Can't execute query: $query. Reason: $DBI::errstr",$Cat,1);
      my $data = $sth -> fetchall_arrayref;
      $sth -> finish;
      my $rows = @{$data};
    
    # ---------------------------------------------------------------
    # Open up a table to surround all boards in this category only if
    # they can view any boards in this category
      if ( ($table_done eq "no") && ($rows) ) {
        if ($j != 0) { print "<br>"; }

        print qq~
          <table border="0" cellpadding="0" cellspacing="0" width="$theme{'tablewidth'}" align="$theme{'tablealign'}" bgcolor="$theme{'catcolor'}">
            <tr>
              <td colspan="2">
                <font size="+1">
                <b>
                $CatTitle
                </b>
                </font>
                <br>
              </td>
            </tr>
            <tr>
              <td width="10">
                &nbsp;
              </td>
              <td>
        ~;
        $table_done = "yes";
        $j = 1;
      }
      elsif ($k == ($BoardCount -1) && (!$rows) ) {
        last;
      }
      elsif (!$rows) {
        next;
      }
      my $Title 	= $data -> [0][0];
      my $Description	= $data -> [0][1]; 
      my $Keyword 	= $data -> [0][2]; 
      my $Total 	= $data -> [0][3]; 
      my $Last		= $data -> [0][4]; 
      my $Number 	= $data -> [0][5]; 
      my $Moderated 	= $data -> [0][6]; 
      my $ReadPerm	= $data -> [0][7];
      my $WritePerm	= $data -> [0][8];
      my $AllowDeny	= $data -> [0][9];
      my $time = "";

    # ------------------------------------------------------------------
    # If Last = 0 then this is a new board so set Last Post to New Board
      if (!$Last){
        $time = "$lang{'NEW_BOARD'}";
      }else{
        $time = w3t::convert_time($Last);
      }

      print qq~
      <table border="0" width="100%" bgcolor="$theme{'tddark'}" cellspacing="0">
        <tr>
          <td width="50%">
            <a href="$config{'cgiurl'}/postlist.pl?Cat=$Cat&Board=$Keyword">$Title</a>
          </td>
          <td width="25%">
      ~;

    # ---------------------------------------------------------------
    # If they are a normal user then they can only see approved posts
      my $Viewable = "And B_Approved = 'yes'";
      if ($user{'Status'} eq "Administrator") {
        $Viewable = "";
      }
      if ($user{'Status'} eq "Moderator") {
      # ---------------------------------
      # Check if they moderate this board
        my $Username_q = $dbh -> quote($Username);
        my $Board_q    = $dbh -> quote($Keyword);
        my $query = qq!
          SELECT Mod_Board
          FROM   w3t_Moderators
          WHERE  Mod_Username = $Username_q
          AND    Mod_Board    = $Board_q
          !;
          my $sth = $dbh -> prepare($query) or w3t::not_right("Can't prepare $query.  Reason: $DBI::errstr.",$Cat,1);
          $sth -> execute() or w3t::not_right("Can't execute $query.  Reason: $DBI::errstr.",$Cat,1);
          my ($check) = $sth -> fetchrow_array;
          $sth -> finish;
          if ($check) {
            $Viewable = "";
          }
        }

      # --------------------------------------------------
      # Check if the are new posts since they last visited
        my $checker;
        my $shownew;
        my $notapproved;
        if ($Username) {
          $Username ||=" ";
          my $Username_q = $dbh -> quote($Username);
          my $Board_q     = $dbh -> quote($Keyword);
          $query = qq!
            SELECT L_Last 
            FROM   w3t_Last  
            WHERE  L_Username = $Username_q
            AND    L_Board    = $Board_q
          !;
          $sth = $dbh -> prepare($query) or w3t::not_right("Query syntax error: $DBI::errstr. Query: $query",$Cat,1);
          $sth -> execute() or w3t::not_right("Can't execute query: $query. Reason: $DBI::errstr",$Cat,1);
          $checker = $sth -> fetchrow_array;
          $sth -> finish;
          $checker ||="0";


        # --------------------------------------------------------------
        # Let's see how many new posts there are.  We have to do some
        # tricky AND/OR operations because we don't want sticky posts to
        # be counted as new if they aren't.
          $query = qq!
            SELECT COUNT(*)
            FROM   w3t_Posts 
            WHERE  ( (B_Posted > $checker AND B_Posted <> 99999999999)
            OR       (B_Posted = 99999999999 AND B_Sticky > $checker) )
            $Viewable 
            AND    B_Board = $Board_q
          !;
          $sth = $dbh -> prepare($query) or w3t::not_right("Query syntax error: $DBI::errstr. Query: $query",$Cat,1);
          $sth -> execute() or w3t::not_right("Can't execute query: $query. Reason: $DBI::errstr",$Cat,1);
          my $newposts = $sth -> fetchrow_array;
          $sth -> finish;

          if ($newposts) {
            $shownew = "<font color=\"$theme{'newfontcolor'}\">($newposts $lang{'NEW_TEXT'})</font>";
          }

        # --------------------------------------
        # Let's see how many NA posts there are
          if ( ($Moderated eq "yes") && ($Viewable eq "") ) {
            $query = qq!
              SELECT COUNT(*)
              FROM   w3t_Posts 
              WHERE  B_Approved = 'no' 
              AND    B_Board    = $Board_q
            !;
            my $sti = $dbh -> prepare($query) or w3t::not_right("Query syntax error: $DBI::errstr. Query: $query",$Cat,1);
            $sti -> execute() or w3t::not_right("Can't execute query: $query. Reason: $DBI::errstr",$Cat,1);
            $notapproved = $sti -> fetchrow_array;
            $sti -> finish;
          }

        }
        if( ($checker < $Last) && ($Username) ) {

          print "<font size=\"-1\">$Total $lang{'POSTS_TEXT'} $shownew";

        } else {

          print "<font size=\"-1\">$Total $lang{'POSTS_TEXT'}";

        }

        if ($notapproved) {
          print " <font color=\"$theme{'newfontcolor'}\">($notapproved $lang{'NOT_APPROVED'})</font>";
        }

        print qq~ 
        </font></td><td width="25%">
        <font size=-1>$lang{'LAST_POST'}: $time</font></td></tr></table>
        <table border="0" width="100%" cellspacing="0">
        <tr><td colspan="3" bgcolor="$theme{'tdlight'}">
        <font size="-1">$Description</font>
        <br>
        </td></tr>
        <tr><td colspan=\"3\">
        <img src=\"$config{'images'}/hr.gif\" alt = \"----------\"></td></tr>
        </table>
        ~;
      }
  
    # ----------------------------------------------------
    # This closes out the table we opened for the category
      if ($table_done eq "yes") {
        print "</td></tr>";
        print "<tr><td bgcolor=\"$theme{'tdalt'}\" colspan=\"2\">";
        print "</td></tr></table>";
      }

      $stb -> finish;
    }

  # Print the footer;
    print "<TABLE BORDER=0 WIDTH=\"$theme{'tablewidth'}\" ALIGN=\"$theme{'tablealign'}\"><TR><TD>";
    print "<p>$lang{'ALL_TIMES'} $config{'timezone'}";
    print "</TD></TR></TABLE>";
    $sta -> finish;
    w3t::send_footer();
